import { Assign, RequiredKeys } from "utility-types";
import { Optionals } from "./types";
declare type Requireds<T> = Pick<T, RequiredKeys<T>>;
export declare const required = "__!!!!!!this is a required property!!!!!!__";
declare type Definition<T> = {
    [K in keyof T]: {} extends Pick<T, K> ? T[K] : typeof required;
};
/**
 * `define<T, U>(props: unknown, definition)`
 * Guarantees at runtime that `props` matches `definition`
 * `U` must only be the optional params on `T`
 *
 * @return {object} T where provided defaults are guaranteed
 *
 * @example
 *
 *     define<{name: string,
 *              highlight?: boolean,
 *              last?: string},
 *           {highlight: boolean}>(unknown_prop, {highlight: false});
 *
 * Unfortunately you must use both type annotations until this goes through
 * https://github.com/microsoft/TypeScript/issues/26242
 *
 **/
export declare function define<T>(props: unknown, definition: Definition<T>): T;
export declare function define<T extends object, U extends Optionals<T>>(props: unknown, definition: Assign<Definition<T>, U>, allow_extra?: boolean, strict?: boolean): Assign<U, Requireds<T>>;
export {};
